#include "MIDIUSB.h"        
#include <ResponsiveAnalogRead.h>  

//deze code is gemaakt met deze tutorial!! (https://www.youtube.com/watch?v=hcm5H6f8MI8)



const int N_POTS = 4; // aantal pots

int potPin[N_POTS] = { A0, A2, A4, A5 }; //welke pins we gebruiken 
int potCC[N_POTS] = { 11, 12, 13, 14 };  //de MIDIcc channels waar we MIDI naar sturen

int potReading[N_POTS] = { 0 }; //directe output van de pots
int potState[N_POTS] = { 0 };   //huyidige staat van de pots
int potPState[N_POTS] = { 0 };  //vorige staat van de pots

int midiState[N_POTS] = { 0 }; //huidige staat van de midi
int ledState[N_POTS] = { 0 };   // staat van de LED-output
int midiPState[N_POTS] = { 0 }; //vorige staat van de midi

const byte potThreshold = 15;   //value deadzone
const int POT_TIMEOUT = 300;    //value change sluis
unsigned long pPotTime[N_POTS] = { 0 }; // laatste keer dat een pot veranderde
unsigned long potTimer[N_POTS] = { 0 }; // tijd sinds laatste pot change

byte POT_CH = 4; // midi channel

float snapMultiplier = 0.01; //snapping voor pot readings
ResponsiveAnalogRead responsivePot[N_POTS] = {};

void setup() {

pinMode(3, OUTPUT);
pinMode(5, OUTPUT);

  Serial.begin(9600);

  for (int i = 0; i < N_POTS; i++) {
    responsivePot[i] = ResponsiveAnalogRead(0, true, snapMultiplier);
    responsivePot[i].setAnalogResolution(1023); //responsiveanalogread dingen
  }
}

void loop() {


  for (int i = 0; i < N_POTS; i++) {

    potReading[i] = analogRead(potPin[i]); //lees pot value
    responsivePot[i].update(potReading[i]); //update de responsive pot
    potState[i] = responsivePot[i].getValue(); //zet de potstate naar de responsivepot value
    midiState[i] = map(potState[i], 0, 1023, 0, 128); //map de potstate(0-1023) naar de midi value (0-128)
    ledState[i] = map(potState[i], 0, 1023, 0, 255); //map de potstate naar de leds

    int potVar = abs(potState[i] - potPState[i]); //absolute verandering in potvalue

    if (potVar > potThreshold) {
      pPotTime[i] = millis(); //moment dat de pot veranderde
    }

    potTimer[i] = millis() - pPotTime[i]; // tijd sinds de pot veranderde

    if (potTimer[i] < POT_TIMEOUT) {
      if (midiState[i] != midiPState[i]) {
        
        controlChange(POT_CH, potCC[i], midiState[i]); //stuur de MIDI-informatie naar de ontvanger
        MidiUSB.flush(); //flush de MIDI-message om hem ook te sturen

        Serial.print("Pot ");
        Serial.print(i);
        Serial.print(" | ");
        Serial.print("PotState: ");
        Serial.print(potState[i]);
        Serial.print(" -  midiState: ");
        Serial.println(midiState[i]);
        midiPState[i] = midiState[i]; //update vorige midi state
        if(i=0){
        analogWrite(3, ledState[i]); //als dit slider 1 is, verander led 1
        }
        else if (i=1){
          analogWrite(5, ledState[i]);// als dit slider 2 is, verander led 2
        }
        
      }
      potPState[i] = potState[i]; // update vorige pot state
    }
  }
}

//MIDIUSB library
void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = { 0x09, 0x90 | channel, pitch, velocity };
  MidiUSB.sendMIDI(noteOn);
}

void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = { 0x08, 0x80 | channel, pitch, velocity };
  MidiUSB.sendMIDI(noteOff);
}

void controlChange(byte channel, byte control, byte value) {
  midiEventPacket_t event = { 0x0B, 0xB0 | channel, control, value };
  MidiUSB.sendMIDI(event);
}